import * as _zag_js_anatomy from '@zag-js/anatomy';
import { RequiredBy, PropTypes, DirectionProperty, CommonProperties, NormalizeProps } from '@zag-js/types';
import * as _zag_js_core from '@zag-js/core';
import { StateMachine } from '@zag-js/core';

declare const anatomy: _zag_js_anatomy.AnatomyInstance<"root" | "label" | "tree" | "item" | "itemIndicator" | "itemText" | "branch" | "branchControl" | "branchTrigger" | "branchContent" | "branchText" | "branchIndicator">;

interface FocusChangeDetails {
    focusedId: string | null;
}
interface ExpandedChangeDetails extends FocusChangeDetails {
    expandedIds: string[];
}
interface SelectionChangeDetails extends FocusChangeDetails {
    selectedIds: string[];
}
interface PublicContext extends DirectionProperty, CommonProperties {
    /**
     * The id of the expanded nodes
     */
    expandedIds: string[];
    /**
     * The id of the selected nodes
     */
    selectedIds: string[];
    /**
     * The id of the focused node
     */
    focusedId: string | null;
    /**
     * Whether the tree supports multiple selection
     * - "single": only one node can be selected
     * - "multiple": multiple nodes can be selected
     *
     * @default "single"
     */
    selectionMode: "single" | "multiple";
    /**
     * Called when the tree is opened or closed
     */
    onExpandedChange?: (details: ExpandedChangeDetails) => void;
    /**
     * Called when the selection changes
     */
    onSelectionChange?: (details: SelectionChangeDetails) => void;
    /**
     * Called when the focused node changes
     */
    onFocusChange?: (details: FocusChangeDetails) => void;
    /**
     * Whether clicking on a branch should open it or not
     * @default true
     */
    openOnClick?: boolean;
}
interface PrivateContext {
}
type ComputedContext = Readonly<{
    /**
     * @computed
     * Whether a typeahead search is ongoing
     */
    isTypingAhead: boolean;
    /**
     * @computed
     * Whether the tree supports multiple selection
     */
    isMultipleSelection: boolean;
}>;
type UserDefinedContext = RequiredBy<PublicContext, "id">;
interface MachineContext extends PublicContext, PrivateContext, ComputedContext {
}
interface MachineState {
    value: "idle";
}
type State = StateMachine.State<MachineContext, MachineState>;
type Send = StateMachine.Send<StateMachine.AnyEventObject>;
interface ItemProps {
    /**
     * The depth of the item or branch
     */
    depth: number;
    /**
     * The id of the item or branch
     */
    id: string;
    /**
     * Whether the item or branch is disabled
     */
    disabled?: boolean;
}
interface BranchProps extends ItemProps {
}
interface ItemState {
    id: string;
    isDisabled: boolean;
    isSelected: boolean;
    isFocused: boolean;
}
interface BranchState extends ItemState {
    isExpanded: boolean;
}
interface MachineApi<T extends PropTypes = PropTypes> {
    /**
     * The id of the expanded nodes
     */
    expandedIds: string[];
    /**
     * The id of the selected nodes
     */
    selectedIds: string[];
    /**
     * Function to expand nodes
     */
    expand(ids: string[]): void;
    /**
     * Function to expand all nodes
     */
    expandAll(): void;
    /**
     * Function to collapse nodes
     */
    collapse(ids: string[]): void;
    /**
     * Function to collapse all nodes
     */
    collapseAll(): void;
    /**
     * Function to select nodes
     */
    select(ids: string[]): void;
    /**
     * Function to select all nodes
     */
    selectAll(): void;
    /**
     * Function to deselect nodes
     */
    deselect(ids: string[]): void;
    /**
     * Function to deselect all nodes
     */
    deselectAll(): void;
    /**
     * Function to focus a branch node
     */
    focusBranch(id: string): void;
    /**
     * Function to focus an item node
     */
    focusItem(id: string): void;
    rootProps: T["element"];
    labelProps: T["element"];
    treeProps: T["element"];
    getItemState(props: ItemProps): ItemState;
    getItemProps(props: ItemProps): T["element"];
    getItemIndicatorProps(props: ItemProps): T["element"];
    getItemTextProps(props: ItemProps): T["element"];
    getBranchState(props: BranchProps): BranchState;
    getBranchProps(props: BranchProps): T["element"];
    getBranchIndicatorProps(props: BranchProps): T["element"];
    getBranchTriggerProps(props: BranchProps): T["element"];
    getBranchControlProps(props: BranchProps): T["element"];
    getBranchContentProps(props: BranchProps): T["element"];
    getBranchTextProps(props: BranchProps): T["element"];
}

declare function connect<T extends PropTypes>(state: State, send: Send, normalize: NormalizeProps<T>): MachineApi<T>;

declare function machine(userContext: UserDefinedContext): _zag_js_core.Machine<MachineContext, MachineState, _zag_js_core.StateMachine.AnyEventObject>;

export { type MachineApi as Api, type BranchProps, type BranchState, type UserDefinedContext as Context, type ExpandedChangeDetails, type FocusChangeDetails, type ItemProps, type ItemState, type SelectionChangeDetails, anatomy, connect, machine };
